/**
 * @file   veil_config.c
 * \code
 *     Author:       Marc Munro
 *     Copyright (c) 2006 Marc Munro
 *     License:      BSD
 * $Id: veil_config.c,v 1.1 2006/10/04 15:50:09 bloodnok Exp $
 * \endcode
 * @brief  
 * Code for setting and reading configuration data.
 * 
 */

#include "postgres.h"
#include "storage/fd.h"
#include "utils/guc.h"
#include "veil_version.h"
#include "veil_funcs.h"


/** 
 * The number of buckets to create in the hash for shared variables.
 * This defaults to 32 and may be defined in postgresql.conf using eg:
 * "veil.shared_hash_elems = 64"
 */
static int shared_hash_elems = 32;

/** 
 * The size in Bytes, of the "chunk" allocation unit.  Veil allocates
 * shared memory in chunks of this size.  If you have large shared
 * memory structures, you may need to increase the size of this.  The
 * default is 8192 Bytes and may be defined in postgresql.conf using eg:
 * "veil.shmem_chunk_size = 8192"
 */
static int shmem_chunk_size = 8192;

/** 
 * The size in KBytes, of each of Veil's shared memory contexts.  Veil
 * will pre-allocate (from Postgres 8.2 onwards) twice this amount of
 * shared memory, one for each context area.
 * The default is 16384 Bytes and may be defined in postgresql.conf
 * using eg: "veil.shmem_context_size = 8192"
 */
static int shmem_context_size = 8192 * 2;

/** 
 * Return the number of entries that should be allocated for shared
 * variables in our shared hashes.  This defaults to 32 and may be
 * defined in postgresql.conf using eg:
 * "veil.shared_hash_elems = 64"
 */
int
veil_shared_hash_elems()
{
	veil_config_init();
	return shared_hash_elems;
}

/** 
 * Return the size of chunk that will be allocated from shared memory
 * when we need more.  This defaults to 8K and may be defined in
 * postgresql.conf using eg: 
 * "veil.shmem_chunk_size = 8192"
 */
int
veil_shmem_chunk_size()
{
	veil_config_init();
	return shmem_chunk_size;
}

/** 
 * Return the amount of shared memory to be requested for each of the
 * two shared memory contexts.  This variable has no effect unless
 * shared_preload_libraries has been defined in postgresql.conf to load
 * the Veil shared library
 * Note that this must be large enough to allocate at least one chunk of
 * memory for each veil-using database in the Postgres cluster.  It
 * defaults to 16K and may be defined in postgresql.conf using eg: 
 * "veil.shmem_context_size = 16384"
 */
int
veil_shmem_context_size()
{
	veil_config_init();
	return shmem_context_size;
}

/** 
 * Initialise Veil's use of GUC variables.
 */
void
veil_config_init()
{
	static bool first_time = true;
	if (!first_time) {
		return;
	}

#if PG_VERSION_GE(8, 2)
	DefineCustomIntVariable("veil.shared_hash_elems",
							"The number of entries for shared variables in "
							"each shared memory context (32)",
							NULL,
							&shared_hash_elems,
							32, 8192,
							PGC_USERSET,
							NULL, NULL);
	DefineCustomIntVariable("veil.shmem_chunk_size",
							"The size of a shared memory chunk (8192 Bytes)",
							NULL,
							&shmem_chunk_size,
							4096, 104857600,
							PGC_USERSET,
							NULL, NULL);
	DefineCustomIntVariable("veil.shmem_context_size",
							"Size of each shared memory context",
							"Size of each shared memory context in bytes.  "
							"This cannot be increased without stopping "
							"and restarting the database cluster.",
							&shmem_context_size,
							4096, 104857600,
							PGC_USERSET,
							NULL, NULL);
#endif

	first_time = false;
}




